/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     | Website:  https://openfoam.org
    \\  /    A nd           | Copyright (C) 2011-2019 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::myTotalFlowRateAdvectiveDiffusiveC3H6O

Description
    This BC is used for species inlets. The diffusion and advection fluxes are
    considered to calculate the inlet value for the species
    The massFluxFraction sets the fraction of the flux of each particular
    species.

SourceFiles
    myTotalFlowRateAdvectiveDiffusiveC3H6O.C

\*---------------------------------------------------------------------------*/

#ifndef myTotalFlowRateAdvectiveDiffusiveC3H6O_H
#define myTotalFlowRateAdvectiveDiffusiveC3H6O_H

#include "mixedFvPatchField.H"
#include "volFields.H"
#include "dimensionedScalar.H"
#include "interpolationTable.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
     Class myTotalFlowRateAdvectiveDiffusiveC3H6O Declaration
\*---------------------------------------------------------------------------*/

class myTotalFlowRateAdvectiveDiffusiveC3H6O
:
    public mixedFvPatchField<scalar>
{
    // Private Data

        //- Name of the flux transporting the field
        word phiName_;

        //- Name of the density field used to normalise the mass flux
        //  if necessary
        word rhoName_;

        //- Mass flux fraction
        scalar massFluxFraction_;

public:

    //- Static table of black body emissive power
    static const List<Tuple2<scalar, scalar>> interpTable;

private:

    // Private Data

        //- Interpolation table of black body emissive power
        mutable interpolationTable<scalar> table_;

public:

    //- Runtime type information
    TypeName("myTotalFlowRateAdvectiveDiffusiveC3H6O");

    // Constructors

        //- Construct from patch and internal field
        myTotalFlowRateAdvectiveDiffusiveC3H6O
        (
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&
        );

        //- Construct from patch, internal field and dictionary
        myTotalFlowRateAdvectiveDiffusiveC3H6O
        (
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&,
            const dictionary&
        );

        //- Construct by mapping given
        //  myTotalFlowRateAdvectiveDiffusiveC3H6O
        //  onto a new patch
        myTotalFlowRateAdvectiveDiffusiveC3H6O
        (
            const myTotalFlowRateAdvectiveDiffusiveC3H6O&,
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&,
            const fvPatchFieldMapper&
        );

        //- Copy constructor
        myTotalFlowRateAdvectiveDiffusiveC3H6O
        (
            const myTotalFlowRateAdvectiveDiffusiveC3H6O&
        );

        //- Construct and return a clone
        virtual tmp<fvPatchField<scalar>> clone() const
        {
            return tmp<fvPatchField<scalar>>
            (
                new
                myTotalFlowRateAdvectiveDiffusiveC3H6O(*this)
            );
        }

        //- Copy constructor setting internal field reference
        myTotalFlowRateAdvectiveDiffusiveC3H6O
        (
            const myTotalFlowRateAdvectiveDiffusiveC3H6O&,
            const DimensionedField<scalar, volMesh>&
        );

        //- Construct and return a clone setting internal field reference
        virtual tmp<fvPatchField<scalar>> clone
        (
            const DimensionedField<scalar, volMesh>& iF
        ) const
        {
            return tmp<fvPatchField<scalar>>
            (
                new
                myTotalFlowRateAdvectiveDiffusiveC3H6O
                (
                    *this,
                    iF
                )
            );
        }


    // Member Functions

        // Mapping functions

            //- Map (and resize as needed) from self given a mapping object
            //  Used to update fields following mesh topology change
            virtual void autoMap(const fvPatchFieldMapper&);

            //- Reverse map the given fvPatchField onto this fvPatchField
            //  Used to reconstruct fields
            virtual void rmap(const fvPatchScalarField&, const labelList&);


        // Evaluation functions

            //- Update the coefficients associated with the patch field
            virtual void updateCoeffs();


        //- Write
        virtual void write(Ostream&) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
